/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Goodger (03/01/00)
 *   Seth Spitzer (28/10/99)
 *   Dan Veditz <dveditz@netscape.com>
 *   Benjamin Smedberg <bsmedberg@covad.net>
 *   Jeffrey Hammel <jhammel@mozilla.com>
 *   Jonathan Griffin <jgriffin@mozilla.com>
 *   Mark Cote <mcote@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//// command line interface

var profileManagerCommand = 'profile-manager'; // TODO: get this dynamically

var gProfileManagerBundle = document.getElementById("bundle_profileManager");

// The program is ran via 
var usage = profileManagerCommand + ' [global options] <command> <command args> [command options]';

function commandLineOption(name, shortName, arguments, help_locale_prop,
                           retvalName, isFile) {
    this.name = name;
    this.shortName = shortName;
    this.arguments = arguments;
    this.help = gProfileManagerBundle.getString(help_locale_prop);
    this.pattern = '-?-' + this.name + '=.*';
    if (retvalName) {
        this.retvalName = retvalName;
    } else {
        this.retvalName = name;
    }
    this.isFile = isFile;

    this.argument = function(commandline) {
        // returns  the rest of the commandline and the argument to the 
        // command (if any); assumes match has already been called

        // no arguments case
        if (!this.arguments) {
            return {commandline: commandline.slice(1)};
        }

        // case of --foo=bar
        if (commandline[0].match(this.pattern)) {
            return {commandline: commandline.slice(1),
                    arg: commandline[0].split('=', 1)[1]};
        }

        // case of --foo bar or -f bar
        if (commandline.length < 2) {
            // check to ensure there is an argument
            throw gProfileManagerBundle.getFormattedString('cliOptTooFewArgsError',
                                                           [commandline[0]]);
        }
        return {commandline: commandline.slice(2),
                arg: commandline[1]}
    }

    this.match = function(arg) {
        if (arg == '-' + this.shortName) {
            return true;
        }
        if ((arg == '--' + this.name) || (arg == '-' + this.name)) {
            return true;
        }
        if (this.arguments) {
            if (arg.match(this.pattern)) {
                return true;
            }
        }
        return false;
    }

    if (this.arguments) {
        this.parse = function(commandline, retval) {
            var data = this.argument(commandline);
            var arg = data.arg;
            if (this.isFile) {
                arg = getFile(arg);
            }
            if (this.process) {
                arg = this.process(arg);
            }
            retval[this.retvalName] = arg;
            return data.commandline;
        }
        
    } else {
        this.parse = function(commandline, retval) {
            var data = this.argument(commandline);
            var arg = true;
            if (this.process) {
                arg = this.process(arg);
            }
            retval[this.retvalName] = arg;
            return data.commandline
        }
    }
}

function commandLineCommand(name, arguments, help_locale_prop) {
    this.name = name;
    this.arguments = arguments;
    this.help = gProfileManagerBundle.getString(help_locale_prop);
    this.usage = this.name;
    for (var i in this.arguments) {
        this.usage += ' ' + this.arguments[i];
    }
    this.parse = function(commandline) {
        this.options = {};
        var index = 0;
        for (var i in commandline) {
             // TODO: add command-specific options
            this.options[this.arguments[index]] = commandline[i];
            index++;
            if (index > this.arguments.length) {
                throw gProfileManagerBundle.getFormattedString('cliCmdTooManyArgsError',
                                                               [this.name]);
            }
        }
        if (index != this.arguments.length) {
            throw gProfileManagerBundle.getFormattedString('cliCmdTooFewArgsError',
                                                           [this.name]);
        }
    }
    this.invoke = function(profileService, defaultApp) {
        // invoke the command, to be implemented by the child class
    }
    this.printCommandUsage = function() {
        dump(gProfileManagerBundle.getString('cliUsageText') + '\n');
        dump(profileManagerCommand + ' ' + this.usage + '\n');
        dump('\n');
        dump(this.help);
        dump('\n');
    }
}


// usage information
function printUsage() {
    dump(gProfileManagerBundle.getString('cliCmdLineText') + ' ' + usage +
         '\n');
    dump(gProfileManagerBundle.getString('cliGuiText') + ' ' +
         profileManagerCommand + ' [options] [app]' + '\n');
    dump('\n' + gProfileManagerBundle.getString('cliUsagePurposeText') + '\n');
    dump('\n' + gProfileManagerBundle.getString('cliUsageDefaultAppText') +
         '\n');

    // TODO: ensure column alignment
    dump('\n' + gProfileManagerBundle.getString('cliUsageGlobalOptsText') +
         '\n');
    var optionNames = new Array(globalOptions.length);
    for (var i in globalOptions) {
        var option = globalOptions[i];
        optionNames[i] = '--' + option.name;
        if(option.arguments) {
            optionNames[i] += '=' + option.arguments;
        }
    }
    optionNames = equalizeLengths(optionNames);
    for (var i in globalOptions) {
        var option = globalOptions[i];
        dump(' ' + optionNames[i]);
        dump('  ' + option.help);
        if(option.shortName) {
            dump(' (-' + option.shortName + ')');
        }
        dump('\n');
    }
    dump('\n' + gProfileManagerBundle.getString('cliUsageCommandsText') +
         '\n');
    var commandNames = new Array(commands.length);
    for(var i in commands) {
        commandNames[i] = commands[i].name;
    }
    commandNames = equalizeLengths(commandNames);
    for(var i in commands) {
        var command = commands[i];
        dump(' ' + commandNames[i] + '  ' + command.help + '\n');
    }
}

// global options
var configOption = new commandLineOption('config', 'c', 
                                         'CONFIG', 'cliOptConfigHelp',
                                         'profileDirectory', true);
configOption.process = function(arg) {
    // TODO: could check to see if a directory or an .ini file is passed 
    // and behave appropriately
    return arg.parent;
}

var applicationOption = new commandLineOption('app', null, 'APPLICATION',
                                              'cliOptApplicationHelp');
var binaryOption = new commandLineOption('binary', 'b', 'BINARYPATH',
                                         'cliOptBinaryHelp', 'binary', true);

var globalOptions = [ configOption, applicationOption, binaryOption ];

// commands
var copyProfileCommand = new commandLineCommand('copy', ['from', 'to'], 
                                                'cliCmdCopyProfileHelp');
copyProfileCommand.invoke = function(profileService, defaultApp) {
    profileService.cloneProfile(this.options['from'],
                                this.options['to']);
}

var deleteProfileCommand = new commandLineCommand('delete', ['profile'],
                                                  'cliCmdDeleteProfileHelp');
deleteProfileCommand.invoke = function(profileService, defaultApp) {
    profileService.deleteProfile(this.options['profile']);
}

var createProfileCommand = new commandLineCommand('create', ['name'],
                                                  'cliCmdCreateProfileHelp');
createProfileCommand.invoke = function(profileService, defaultApp) {
    profileService.newProfile(this.options.name, false);
}

var listProfilesCommand = new commandLineCommand('list', [],
                                                 'cliCmdListProfilesHelp');
listProfilesCommand.invoke = function(profileService, defaultApp) {
     for (var i in profileService.profiles) {
         var profile = profileService.profiles[i];
         dump(profile.name + '\n');
     }
}

var profileBackupsCommand = new commandLineCommand('backups', [],
                                                   'cliCmdProfileBackupsHelp');
profileBackupsCommand.parse = function(commandline) {
    this.profile = null;
    if (commandline.length == 1) {
        this.profile = commandline[0];
    } else if (commandline.length > 1) {
        throw gProfileManagerBundle.getString('cliCmdProfileBackupsTooManyError');
    }
}
profileBackupsCommand.invoke = function(profileService, defaultApp) {
    if (this.profile) {
        var profile = profileService.getProfile(this.profile);
        for (var j in profile.Backups) {
            dump(profile.Backups[j].date + '\n');
        }
    } else {
        for (var i in profileService.profiles) {
            var profile = profileService.profiles[i];

            if (!profile.Backups.length) { continue; }

            dump(profile.name + ':\n');
            for (var j in profile.Backups) {
                var backup = profile.Backups[j];
                dump(' ' + backup.date + '\n');
            }
            if (i != profileService.profiles.length) {
                dump('\n');
            }
        }
    }
}
profileBackupsCommand.usage = 'backups [profile]';

var profileBackupCommand = new commandLineCommand('backup', ['profile'],
                                                  'cliCmdProfileBackupHelp');
profileBackupCommand.invoke = function(profileService, defaultApp) {
    profileService.backupProfile(this.options.profile);
    var profile = profileService.getProfile(this.options.profile);
    dump(profile.Backups[0].file.path + '\n');
}

var restoreProfileCommand = new commandLineCommand('restore', ['profile'],
                                                   'cliCmdRestoreProfileHelp');
restoreProfileCommand.invoke = function(profileService, defaultApp) {
  // TODO: allow restoration with a specified --date
  profileService.restoreProfile(this.options.profile);
}

var helpCommand = new commandLineCommand('help', ['command'],
                                         'cliCmdHelpHelp');
helpCommand.parse = function(commandline) {
    this.command = null;
    if (commandline.length == 1) {
        this.command = getCommand(commandline[0]);
        if (!this.command) {
            throw gProfileManagerBundle.getFormattedString('cliCmdHelpInvalidCmdError',
                                                           [commandline[0]]);
        }
    } else if (commandline.length > 1) { 
        throw gProfileManagerBundle.getString('cliCmdHelpTooManyCmdsError');
    }
}
helpCommand.invoke = function(profileService, defaultApp) {
    if (this.command) {
        this.command.printCommandUsage();
    } else {
        printUsage();
    }
}
var profileInfoCommand = new commandLineCommand('info', ['profile'],
                                                'cliCmdProfileInfoHelp');
profileInfoCommand.invoke = function(profileService, defaultApp) {

    var profile = profileService.getProfile(this.options.profile);
    if (!profile) {
        throw gProfileManagerBundle.getString('cliCmdProfileInfoNoProfileGivenError');
    }
    var keys = new Array();
    var values = new Array();

    function pushLocaleStrKey(locale_prop) {
      keys.push(gProfileManagerBundle.getString(locale_prop));
    };

    // sequester the information
    pushLocaleStrKey('cliProfileNameText');
    values.push(profile.name);
    pushLocaleStrKey('cliProfilePathText')
    values.push(profile.rootDir.path);
    pushLocaleStrKey('cliProfileBackupsText');
    values.push(profile.Backups.length);
    pushLocaleStrKey('cliProfileDefaultText')
    if (profile.Default) {
        values.push('true');
    } else {
        values.push('false');
    }
    pushLocaleStrKey('cliProfileSectionText');
    values.push(profile.section);

    // format the keys 
    keys = equalizeLengths(keys);

    // print the data
    for (var i in keys) {
        dump(keys[i] + ' ' + values[i] + '\n');
    }
}


var commands = [ profileBackupCommand, 
                 profileBackupsCommand, 
                 copyProfileCommand, 
                 createProfileCommand,
                 deleteProfileCommand,
                 helpCommand,
                 profileInfoCommand,
                 listProfilesCommand,
                 restoreProfileCommand]; 
// TODO: other commands to add
// - restore ... a profile from backup
// - backup ... a profile
// - rename ... a profile
// - default ... make a profile the default, or display the default
// - check ... check profiles for errors

function getCommand(name) {
    for (var i in commands) {
        if (name == commands[i].name) {
            return commands[i];
        }
    }
}

function parseCommandline(_commandline) {

    var result = {app: 'firefox'}; // defaults

    // turn the nsICommandLine into an array
    // since we're rolling our own anyways
    var commandline = new Array(_commandline.length);
    for (var i=0; i < _commandline.length; i++) {
        commandline[i] = _commandline.getArgument(i);
    }

    // parse global options until we hit a command
    var foundOption;
    var foundApp = false;
    while (commandline.length) {
        foundOption = false;
        var arg = commandline[0];
        
        var command = getCommand(arg);
        if (command) {
            try {
                command.parse(commandline.slice(1));
            } catch(e) {
                dump(e + '\n\n');
                command.printCommandUsage();
                throw e;
            }
            result.command = command;
            return result;
        }

        for (var i in globalOptions) {
            var option = globalOptions[i];
            if (option.match(arg)) {
                try {
                    commandline = option.parse(commandline, result);
                } catch(e) {
                    // TODO: print usage
                    dump(e + '\n');
                    // TODO: exit with status code 1
                    throw e;
                }
              
                foundOption = true;
            }
        }
        if (foundOption) { continue; }

        // set the application
        // XXX to deprecate?
        // XXX note that this does NOT check redundancy with --app :(
        if(foundApp) {
            throw gProfileManagerBundle.getString('cliTooManyArgsError');
            // TODO: print usage (help command)
        }
        result.app = arg;
        commandline = commandline.slice(1);
    }

    return result;
}
