/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Profile Manager.
 *
 * The Initial Developer of the Original Code is
 * the Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Jonathan Griffin <jgriffin@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * Extends a base class with a sub class.
 */
function extend(subClass, baseClass) {
  function inheritance() {}
  inheritance.prototype = baseClass.prototype;

  subClass.prototype = new inheritance();
  subClass.prototype.constructor = subClass;
  subClass.baseConstructor = baseClass;
  subClass.superClass = baseClass.prototype;
}

/**
 * Create a progres window that contains a progress meter which is
 * visible during lengthy operations.
 * @param opener - the window that the progress meter will belong to
 * @param description - a one-line description of what's happening
 */
function progressMeter(opener, description) {
  this.win = null;
  // specify an onshow method to be called when onload is complete
  this.onshow;
  // specify an oncancel method to be called when the user clicks cancel
  this.oncancel;
  this.opener = opener;
  this.description = description;
  this.busyWindow = false;
}

/**
 * Create a busy window that displays a static message during
 * lengthy operations.
 * @param opener - the window that the progress meter will belong to
 * @param description - a one-line description of what's happening
 */
function busyWindow(opener, description) {
  busyWindow.baseConstructor.call(this, opener, description);
  this.busyWindow = true;
}
extend(busyWindow, progressMeter);

progressMeter.prototype.cancel = function() {
  if (this.oncancel)
    this.oncancel();
}

progressMeter.prototype.show = function() {
  this.win = window.open("chrome://profilemanager/content/progressMeter.xul",
                         "progressWindow",
                         "chrome,dependent=yes");
  var that = this;
  this.win.addEventListener('load', function() { that.onload(); }, false);
};

progressMeter.prototype.close = function() {
  this.win.close();
}

progressMeter.prototype.onload = function() {
  var that = this;

  if (this.busyWindow) {
    this.win.document.getElementById('progress-box').
                      setAttribute('hidden', 'true');
  }
  else {
    this.win.document.getElementById('busy-image').
                      setAttribute('hidden', 'true');
    this.win.document.getElementById('busy-separator').
                      setAttribute('hidden', 'true');
    var cancelButton = this.win.document.getElementById('button-cancel');
    cancelButton.addEventListener('command',
                                  function() { that.cancel(); },
                                  false);
  }

  if (this.description) {
    let desc = this.win.document.getElementById('description');
    desc.appendChild(this.win.document.createTextNode(this.description));
  }

  // size to content, so that outerWidth/outerHeight are accurate
  this.win.sizeToContent();

  // calculate new offsets based on parent's current position and size
  var xOffset = this.opener.screenX + this.opener.outerWidth/2 -
                this.win.outerWidth/2;
  var yOffset = this.opener.screenY + this.opener.outerHeight/2 -
                this.win.outerHeight/2;

  xOffset = ( xOffset > 0 ) ? xOffset : 0;
  yOffset = ( yOffset > 0 ) ? yOffset : 0;
  this.win.moveTo( xOffset, yOffset);

  if (this.onshow) {
    // give the meter a minimum amount of time to be visible, for very short 
    // operations; otherwise the window can flicker in and out of existence
    // unattractively
    setTimeout(function() { that.onshow(); }, 1000);
  }
}